/* Denker's Desk Calculator */
/* this is the LEX script for parsing the input to ddc */

alpha   [a-zA-Z]
alpha_   [a-zA-Z_]
num     [0-9]
hex     [0-9a-fA-F]
al_num  [a-zA-Z_0-9]
exp     ([eE][-+]?{num}+)
U       [\x80-\xbf]
U2      [\xc2-\xdf]
U3      [\xe0-\xef]
U4      [\xf0-\xf4]
UANY    {ASC}|{U2}{U}|{U3}{U}{U}|{U4}{U}{U}{U}
UONLY   {U2}{U}|{U3}{U}{U}|{U4}{U}{U}{U}
/* hat tip: http://stackoverflow.com/questions/9611682/flexlexer-support-for-unicode */
%%
                        int     idx;

"/--".*"\n"             {
/*                        yytext[yyleng-1] = 0; */
                          yyless(yyleng-1);
                          yylval.strval = yytext+3;
                          return(ECHOTXT);
                        }

{alpha_}{al_num}*        {
        /* fprintf(stderr, "saw identifier: %s\n", yytext); */
                          /* first, see if is a builtin symbol */
                          /* the ival is an index into bname and btoktab */
                          yylval.ival = idx = builtup(yytext);
                          /* and the return value is a category e.g. Lib1 or Lib2 */
                          if (idx >= 0) return(btoktab[idx]);
                          /* must be a user symbol of some kind */
                          /* allocate a temporary symbol */
                          if (Ttop >= maxU) {
                            fprintf(stderr, "?TempNamespace overflow\n");
                            return(BOGUS); /* error */
                          }
                          yylval.symval = Tsym[Ttop++] = create(yytext, yyleng);

                          yylval.symval->ndx = idx = bindup(yytext);
                          if (idx >= 0) return(ANAME);
                        /* here if NOT a formal argument */
                          yylval.symval->ndx = idx = lookup(yytext);
                          if (idx < 0) return(XNAME);
                          if (Usym[idx]->syn == ISVAR) return(VNAME);
                          if (Usym[idx]->syn >= 0) return(FNAME);
                          return(BOGUS);  /* compiler error -- unrecognized syntax code */
                        }
"0x"{hex}*              |
{num}+{exp}?            |
{num}*"."{num}+{exp}?   |
{num}+"."{num}*{exp}?   { errno = 0;
                          yylval.dval = strtod(yytext, 0);
                          if (errno){
                            printf("?Unscannable number '%s'\n", yytext);
                          }
                          return(NUMBER);
                        }
"/""/".*$               /* ignore comment */;
"/""*".*"*""/"          /* ignore comment */;

"??"                    return(QQ);
"**"                    return(POW);
"=="                    return(EQ);
"<>"                    return(NEQ);
"!="                    return(NEQ);
"<="                    return(LEQ);
">="                    return(GEQ);
"+="                    return(P_ASS);
"-="                    return(M_ASS);
"*="                    return(T_ASS);
"/="                    return(D_ASS);
[ \t]+                  /* ignore whitespace */;
"√"                     {
                          /* Cute unicode math operator. */
                          /* The parser knows nothing of unicode */
                          /* but can treat UTF-8 as multi-byte strings. */
                          yylval.ival = builtup("sqrt");
                          return(Lib1);
                        }
"÷"                     return('/');
"×"                     return('*');
"\n"|.                  return(yytext[0]) /* other single char represents itself: */;
{UONLY}                 {
/* Apparently this rule has to be here; */
/* otherwise the low-level flex(lex)/bison library code */
/* immediately exits on any unicode character. */
/* Don't ask me to explain its behavior. */
                          return(ODDBALL);
                        }
